"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndySdkSovDidRegistrar = void 0;
const core_1 = require("@aries-framework/core");
const error_1 = require("../error");
const indyError_1 = require("../error/indyError");
const ledger_1 = require("../ledger");
const types_1 = require("../types");
const did_1 = require("../utils/did");
const didSovUtil_1 = require("./didSovUtil");
class IndySdkSovDidRegistrar {
    constructor() {
        this.supportedMethods = ['sov'];
    }
    async create(agentContext, options) {
        var _a, _b;
        const indySdkPoolService = agentContext.dependencyManager.resolve(ledger_1.IndySdkPoolService);
        const didRepository = agentContext.dependencyManager.resolve(core_1.DidRepository);
        const { alias, role, submitterVerificationMethod, indyNamespace } = options.options;
        const privateKey = (_a = options.secret) === null || _a === void 0 ? void 0 : _a.privateKey;
        if (privateKey && !(0, core_1.isValidPrivateKey)(privateKey, core_1.KeyType.Ed25519)) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: 'Invalid private key provided',
                },
            };
        }
        if (!submitterVerificationMethod.startsWith('did:sov:')) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: 'Submitter did must be a valid did:sov did',
                },
            };
        }
        try {
            const signingKey = await agentContext.wallet.createKey({
                privateKey,
                keyType: core_1.KeyType.Ed25519,
            });
            const verkey = signingKey.publicKeyBase58;
            const unqualifiedIndyDid = (0, did_1.indyDidFromPublicKeyBase58)(verkey);
            // FIXME: we should store the didDocument in the DidRecord so we don't have to fetch our own did
            // from the ledger to know which key is associated with the did
            const didsApi = agentContext.dependencyManager.resolve(core_1.DidsApi);
            const didResult = await didsApi.resolve(submitterVerificationMethod);
            if (!didResult.didDocument) {
                return {
                    didDocumentMetadata: {},
                    didRegistrationMetadata: {},
                    didState: {
                        state: 'failed',
                        reason: `didNotFound: unable to resolve did ${submitterVerificationMethod}: ${didResult.didResolutionMetadata.message}`,
                    },
                };
            }
            const verificationMethod = didResult.didDocument.dereferenceKey(submitterVerificationMethod);
            const submitterSigningKey = (0, core_1.getKeyFromVerificationMethod)(verificationMethod);
            const qualifiedSovDid = `did:sov:${unqualifiedIndyDid}`;
            const [unqualifiedSubmitterDid] = submitterVerificationMethod.replace('did:sov:', '').split('#');
            const pool = indySdkPoolService.getPoolForNamespace(indyNamespace);
            await this.registerPublicDid(agentContext, unqualifiedSubmitterDid, submitterSigningKey, unqualifiedIndyDid, signingKey, alias, pool, role);
            // Create did document
            const didDocumentBuilder = (0, didSovUtil_1.sovDidDocumentFromDid)(qualifiedSovDid, verkey);
            // Add services if endpoints object was passed.
            if (options.options.endpoints) {
                await this.setEndpointsForDid(agentContext, unqualifiedIndyDid, signingKey, options.options.endpoints, pool);
                (0, didSovUtil_1.addServicesFromEndpointsAttrib)(didDocumentBuilder, qualifiedSovDid, options.options.endpoints, `${qualifiedSovDid}#key-agreement-1`);
            }
            // Build did document.
            const didDocument = didDocumentBuilder.build();
            const didIndyNamespace = pool.config.indyNamespace;
            const qualifiedIndyDid = `did:indy:${didIndyNamespace}:${unqualifiedIndyDid}`;
            // Save the did so we know we created it and can issue with it
            const didRecord = new core_1.DidRecord({
                did: qualifiedSovDid,
                role: core_1.DidDocumentRole.Created,
                tags: {
                    recipientKeyFingerprints: didDocument.recipientKeys.map((key) => key.fingerprint),
                    qualifiedIndyDid,
                },
            });
            await didRepository.save(agentContext, didRecord);
            return {
                didDocumentMetadata: {
                    qualifiedIndyDid,
                },
                didRegistrationMetadata: {
                    didIndyNamespace,
                },
                didState: {
                    state: 'finished',
                    did: qualifiedSovDid,
                    didDocument,
                    secret: {
                        // FIXME: the uni-registrar creates the seed in the registrar method
                        // if it doesn't exist so the seed can always be returned. Currently
                        // we can only return it if the seed was passed in by the user. Once
                        // we have a secure method for generating seeds we should use the same
                        // approach
                        privateKey: (_b = options.secret) === null || _b === void 0 ? void 0 : _b.privateKey,
                    },
                },
            };
        }
        catch (error) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: `unknownError: ${error.message}`,
                },
            };
        }
    }
    async update() {
        return {
            didDocumentMetadata: {},
            didRegistrationMetadata: {},
            didState: {
                state: 'failed',
                reason: `notImplemented: updating did:sov not implemented yet`,
            },
        };
    }
    async deactivate() {
        return {
            didDocumentMetadata: {},
            didRegistrationMetadata: {},
            didState: {
                state: 'failed',
                reason: `notImplemented: deactivating did:sov not implemented yet`,
            },
        };
    }
    async registerPublicDid(agentContext, submitterDid, submitterSigningKey, targetDid, signingKey, alias, pool, role) {
        const indySdk = agentContext.dependencyManager.resolve(types_1.IndySdkSymbol);
        const indySdkPoolService = agentContext.dependencyManager.resolve(ledger_1.IndySdkPoolService);
        try {
            agentContext.config.logger.debug(`Register public did '${targetDid}' on ledger '${pool.didIndyNamespace}'`);
            const request = await indySdk.buildNymRequest(submitterDid, targetDid, signingKey.publicKeyBase58, alias, role || null);
            const response = await indySdkPoolService.submitWriteRequest(agentContext, pool, request, submitterSigningKey);
            agentContext.config.logger.debug(`Registered public did '${targetDid}' on ledger '${pool.didIndyNamespace}'`, {
                response,
            });
            return targetDid;
        }
        catch (error) {
            agentContext.config.logger.error(`Error registering public did '${targetDid}' on ledger '${pool.didIndyNamespace}'`, {
                error,
                submitterDid,
                targetDid,
                verkey: signingKey.publicKeyBase58,
                alias,
                role,
                pool: pool.didIndyNamespace,
            });
            throw error;
        }
    }
    async setEndpointsForDid(agentContext, did, signingKey, endpoints, pool) {
        const indySdk = agentContext.dependencyManager.resolve(types_1.IndySdkSymbol);
        const indySdkPoolService = agentContext.dependencyManager.resolve(ledger_1.IndySdkPoolService);
        try {
            agentContext.config.logger.debug(`Set endpoints for did '${did}' on ledger '${pool.didIndyNamespace}'`, endpoints);
            const request = await indySdk.buildAttribRequest(did, did, null, { endpoint: endpoints }, null);
            const response = await indySdkPoolService.submitWriteRequest(agentContext, pool, request, signingKey);
            agentContext.config.logger.debug(`Successfully set endpoints for did '${did}' on ledger '${pool.didIndyNamespace}'`, {
                response,
                endpoints,
            });
        }
        catch (error) {
            agentContext.config.logger.error(`Error setting endpoints for did '${did}' on ledger '${pool.didIndyNamespace}'`, {
                error,
                did,
                endpoints,
            });
            throw (0, indyError_1.isIndyError)(error) ? new error_1.IndySdkError(error) : error;
        }
    }
}
exports.IndySdkSovDidRegistrar = IndySdkSovDidRegistrar;
//# sourceMappingURL=IndySdkSovDidRegistrar.js.map